#include <stdio.h>

do_fio(){printf("do_fio\n");} 
e_wsfe(){printf("e_wsfe\n");}
s_stop(){printf("s_stop\n");}
s_wsfe(){printf("s_wsfe\n");} 
err2090_(){ printf("err2090 Improper input parameter values\n");}
err2091_(){ printf("err2091  All colinear data points, or identical points\n");}
err2091b_(){ printf("err2091b All colinear data points, or identical points\n");}
err2092_(){ printf("err2092 error detected in routine idcldp \n");}
err2093_(){ printf("err2093 error detected in routine   idtang \n");}
err2094_(){ printf("err2094 This error is never called.\n");}
