#include <curses.h>

window(int begin_y, int begin_x, int numlines, int numcols)
{}

clreol()
{
}
clrscr()
{
}
cputs(char *line)
{
}
delline()
{
}
gotoxy(int x, int y)
{
}
insline()
{
}
putch(int char_val)
{
}
scr_refresh()
{
}
int scr_getch()
{
}
scr_init()
{
}
scr_end()
{
}
textbackground(int color_num)
{}
textcolor(int colornum)
{}
