enum {	tp_parskip=1, tp_char, 	tp_def, 	tp_defbegin,
	tp_delcode, 	tp_delimiter, 	tp_frac, 	tp_left,
	tp_lineskip, 	tp_mathchar, 	tp_mathchardef, tp_mathcode,
	tp_movexy, 	tp_nolimits, 	tp_overbrace, 	tp_overline,
	tp_right, 	tp_setfont, 	tp_sethei,	tp_sfont,
	tp_ssfont,	tp_sub,		tp_sup,		tp_tfont,
 	tp_underbrace, 	tp_underline,	tp_presave,	tp_chardef,
	tp_newline,	tp_hfill,	tp_setstretch,	tp_linegap,
	tp_rule,	tp_accent};






