# This procedure coppies the necessary files to the system 
# areas.


# Get rid of old versions.
rm /usr/local/bin/gle_*
rm /usr/local/bin/gle
rm /usr/local/bin/surface
rm /usr/local/bin/fitls
rm /usr/local/bin/fitz
rm /usr/local/bin/letz
rm /usr/local/bin/manip
rm /usr/local/bin/contour
rm /usr/local/bin/surf_*


# Copy the font and other data files in.
mkdir /usr/local/gle
cp *.hlp 	/usr/local/gle
cp graph.idx    /usr/local/gle
cp inittex.ini  /usr/local/gle
mkdir /usr/local/gle/font
cp font/*.fve 	/usr/local/gle/font
cp font/*.fmt 	/usr/local/gle/font
cp font/font.dat /usr/local/gle/font
mkdir /usr/local/gle/demo
cp demo/* 	/usr/local/gle/demo

# Copy the binaries in.
cp gle_* 		/usr/local/gle
cp gle 			/usr/local/gle
cp surface 		/usr/local/gle
cp util/fitls/fitls 	/usr/local/gle
cp util/fitz/fitz 	/usr/local/gle
cp util/letz/letz 	/usr/local/gle
cp util/manip/manip 	/usr/local/gle
cp util/contour/contour /usr/local/gle
cp util/surf/surf_* 	/usr/local/gle


# Link to a directory which should be in the path.
ln -s /usr/local/gle/gle /usr/local/bin/gle
ln -s /usr/local/gle/surface /usr/local/bin/surface
ln -s /usr/local/gle/fitls /usr/local/bin/fitls
ln -s /usr/local/gle/fitz /usr/local/bin/fitz
ln -s /usr/local/gle/letz /usr/local/bin/letz
ln -s /usr/local/gle/manip /usr/local/bin/manip
ln -s /usr/local/gle/contour /usr/local/bin/contour


