#include "rgb.h"

	struct gmodel {
		double image[3][3];
		double fontn,fontsz;	/* up to here for font cacheing*/
		colortyp color,fill;
		double lwidth,lstyled,curx,cury;
		double endx,endy;
		double miterlimit;
		int lcap,ljoin;
		int just,xinline,inpath,npath; /* up to here for STATE */
		char lstyle [9];
	/* 18*8, 6*int, 2*int32 */

		double xmin,xmax,ymin,ymax;	/* bounds in USER coordinates */
		double startx,starty;
		double closex,closey;	/* for closepath */
		double userwidth,userheight; /* The user req size */
		double devwidth,devheight;
	} ;

#define SIZEOFSTATE (18*sizeof(double) + 6*sizeof(int) + 2*sizeof(int32) + 9)














