	 Have you read INSTALL, defined GLE_TOP, deleted inittex.ini
	 and touched the gle tree so that make will understand the dates.
              setenv GLE_TOP "/usr/chris/gle/"
              (or) souce path.csh
              (or) . path.sh
            touch *
            touch */*
            touch */*/*
	 Copy  config.machine to config.i  (sun,aix,ultrix)
	    cp config.sun config.i
	 Do all of these before running MAKE the first time.
	 Delete inittex.ini every time you change init.tex
	    rm inittex.ini



