#include "all.h"
#define BEGINDEF
#include "begin.h"
#define true (!false)
#define false 0

begin_token(int32 **pcode,int *cp,int *pln,char *srclin,char *tk,int *ntk,char *outbuff)
{
	if (*((*pcode)+*cp-1)==0) { /* back step a line */
		(*pln)--;
		return false;
	}
	strcpy(srclin,(char *) ((*pcode)+*cp));
	(*pcode) = (*gpcode)[(*pln)++]; *cp = 0;
	token(srclin,(TOKENS) tk,ntk,outbuff);
	return true;
}
begin_init()
{
    int i;
    strcpy(space_str," ");
    if (tk[499]==NULL) for (i=0;i<500;i++) tk[i] = space_str;
}
begin_next_line(int32 *pcode, int *cp)
{
	if (*(pcode+*cp)!=5) { /* END XXX */
		gprint("Expecting another text in pcode (5) , got %ld \n"
			,*(pcode+*cp));
		return false;
	}
	(*cp) += 2;
	return true;
}




















