#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#ifndef unix
#include <stdarg.h>
#endif 
#ifdef __TURBOC__
#include <alloc.h>
#endif
#define false (0)
#define true (!false)
#ifdef unix
#define farmalloc malloc
#endif
#ifdef VAXC
#define farmalloc malloc
char *strdup(char *s);
#endif
#ifndef unix
void gprint(va_list arg_list, ...); 
#endif
FILE *myfopen(char *fname, char *mode);
int read_data(char *infile,float z[], int zdim,float *zmin,float *zmax,int *nx,int *ny);

