#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <stdarg.h>
main()
{
	double  z,xx,yy;
	int x,y;
	int nx=300, ny=300;

	printf("! nx %d ny %d \n",nx,ny);
	for (x=0; x<nx; x++) {
		for (y=0;y<ny; y++) {
			xx = x/6.0; yy = y/6.0;
			z = 3.0/2.0*(cos(3./5.*(yy-1))+5./4.) / (1.+( pow(((xx-4.)/3.),2)  ));
			printf("%g ",z);
		}
		printf("\n");
	}
	printf("\n");
}
