#define COLOR_CLEAR 0XFF000000
#define COLOR_BLACK 0X01000000
#define COLOR_WHITE 0X01FFFFFF
#define COLOR_RED 0X01FF0000
#define COLOR_GREEN 0X0100FF00
#define COLOR_BLUE 0X010000FF
#define COLOR_GREY 0X017F7F7F














 
