#include "all.h"
#include "edt.h"
#ifdef __TURBOC__
#define DASHCHAR 249
#include <alloc.h>
#include "bios.h"
#include "conio.h"
#include "dir.h"
#else
#define DASHCHAR '.'
#include "vaxconio.h"
#endif

#define true (!false)
#define false 0
#define BCOLOR BLUE
#define FCOLOR MAGENTA
/* function prototypes */

/*
manip /com=a.a

	refresh screen.
	edit cell mode,
	command mode.
	arrow keys scroll screen.

	data cell:range
	cell:range = expression
	expressions,   	c,r,x,y,c1..cn,r1..rn,sum(range),
			c(x-1,y) = c(x,y)

*/
/* global stuff */
int gle_debug;
int moving_x,moving_y;
int data_x;
int data_y;
int changed;
extern int islogging;
extern int isating;
/* local stuff */
int iserr,dont_clear;
int scr_frow=1,scr_fcol=1;
int scr_width=76,scr_colwidth=8;
int scr_ncol;
int scr_nrow=18;
int curx,cury;
int max_x=0,max_y=0; 	/* change to 1,1 */

int scr_blackwhite;
int exit_manip;
char file_name[80];
int in_recover,single_step;
/*---------------------------------------------------------------------------*/
main(int argc, char **argv)
{
	int i;
	char ans[90];
	int cmd;

	printf("dopasscmd, atof {%s} %g \n","123",atof("123")); getch();
}
