int v_open(float width, float height);
int v_close(void);
int v_lstyle(char *s);
int v_set_just(char *s);
int v_line(float zx,float zy);
int v_move(float zx,float zy);
int v_color(char *s);
int v_gsave(void);
int v_grestore(void);
int v_text(char *s);
int v_marker(char *s);
int v_set_hei(float g);
int v_rotate(float g);
