enum {	k_save=1, k_load, k_quit, k_exit, k_if, k_for, k_next, k_end
	, k_call, k_else, k_print, k_input, k_generate, k_at, k_comment
	, k_let, k_copy, k_move, k_set ,k_delete ,k_data ,k_insert
	,k_goto ,k_logging ,k_prop ,k_sort ,k_swap, k_close, k_clear
	,k_help, k_list, k_sum, k_parsum, k_shell, k_new, k_fit};
