struct axis_struct {
	int type;	/* 1=xaxis 2=yaxis 3=x2axis 4=y2axis */
	double base;
	double length;
	double shift;
	int32 color;
	int label_font;
	double label_hei;
	double label_scale;
	double label_dist;
	int log,lg1,lg25,lgset;	/* 0=normal 1=log, lg*=small labelling values */
	double min,max;
	int minset,maxset;
	int nofirst,nolast;
	int nticks,nsubticks;
	int decimal;		/* number of decimal places for labels */
	double dticks,dsubticks;
	double ticks_length,ticks_scale,ticks_lwidth;
	char ticks_lstyle[9];
	double subticks_length,subticks_scale,subticks_lwidth;
	char subticks_lstyle[9],label_lstyle[9];
	int off;
	int label_off;
	int side_off,ticks_off,subticks_off;
	double side_lwidth,label_lwidth;
	char side_lstyle[9];
	int title_font;
	int32 ticks_color,side_color,subticks_color,label_color;
	double title_dist;
	double title_hei,title_scale;
	int title_rot,title_off;
	int32 title_color;	/* 0=normal, 1=rotate 180 */
	char *title;
	char *names[50];
	double places[50];
	int nplaces,nnam;
	int negate;
} ;

