#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <assert.h>
#ifndef unix
#include <stdarg.h>
#endif
#include "glepro.h"

#ifdef EMXOS2           /* a.r. */
#define clreol  v_clreol
#define putch   v_putc
#define cputs   v_puts
#endif
