#define  JUST_LEFT 0x100
#define  JUST_CENTER 0x110
#define  JUST_CENTRE 0x110
#define  JUST_RIGHT 0x120
#define  JUST_LT	 0x02
#define  JUST_TL	 0x02
#define  JUST_CL	 0x01
#define  JUST_LC	 0x01
#define  JUST_BL	 0x00
#define  JUST_LB	 0x00
#define  JUST_CB	 0x10
#define  JUST_BC	 0x10
#define  JUST_RB	 0x20
#define  JUST_BR	 0x20
#define  JUST_CR	 0x21
#define  JUST_RC	 0x21
#define  JUST_TR	 0x22
#define  JUST_RT	 0x22
#define  JUST_TC	 0x12
#define  JUST_CT	 0x12
#define  JUST_CC	 0x11
#define  JUST_C	 0x11















 
