src=gle
cd ..
rm gle.tar
rm gle.tar.gz
tar -cvf gle.tar $src/INSTALL
tar -rvf gle.tar $src/*.sh
tar -rvf gle.tar $src/*.csh
tar -rvf gle.tar $src/*.com
tar -rvf gle.tar $src/*.txt
tar -rvf gle.tar $src/config.*
tar -rvf gle.tar $src/*.c
tar -vrf gle.tar $src/*.h
tar -vrf gle.tar $src/*.hlp
tar -rvf gle.tar $src/init.tex
tar -rvf gle.tar $src/makefile*
tar -rvf gle.tar $src/Makefile*
tar -rvf gle.tar $src/def.def
tar -vrf gle.tar $src/demo/*.gle
tar -vrf gle.tar $src/demo/*.sur
tar -vrf gle.tar $src/demo/*.dat
tar -vrf gle.tar $src/demo/*.z
tar -vrf gle.tar $src/demo/*.let
tar -rvf gle.tar $src/util/*/*.c
tar -rvf gle.tar $src/util/*/*.hlp
tar -rvf gle.tar $src/util/*/*.h
tar -rvf gle.tar $src/util/*/Makefile*
tar -rvf gle.tar $src/util/*/makefile*
tar -rvf gle.tar $src/util/*/def.def
tar -rvf gle.tar $src/util/*/*.let
tar -rvf gle.tar $src/util/*/*.z
tar -rvf gle.tar $src/util/*/*.gle
tar -rvf gle.tar $src/util/*/*.dat
tar -rvf gle.tar $src/util/surf/edt.old
tar -rvf gle.tar $src/linux/*
tar -rvf gle.tar $src/ps-docs/*
tar -rf gle.tar $src/font/*.afm
tar -rf gle.tar $src/font/*.gle
tar -rvf gle.tar $src/font/*.dat
tar -rvf gle.tar $src/font/Makefile*
tar -rvf gle.tar $src/font/makefile*
tar -rvf gle.tar $src/manual/gle.txt
tar -rvf gle.tar $src/os2/readme.os2
tar -rvf gle.tar $src/gle32/gle32doc
tar -rvf gle.tar $src/makewarn.os2
tar -rvf gle.tar $src/makewarn.dj
tar -rvf gle.tar $src/makos2.cmd
tar -rvf gle.tar $src/makgle32.bat
tar -rvf gle.tar $src/readme.os2

echo Compressing gle.tar
gzip gle.tar
rm gle33g.tar.gz
mv gle.tar.gz gle33g.tar.gz
cd $src
