/* THIS IS A DUMMY VERSION OF THE SUB.C STUFF */

#include "all.h"
struct sub_st {char name[40];int typ; int np
		; int ptyp[20]; char *pname[20]; int start; int end ; }  ;
struct sub_st *sb[100];
int nsb;
double return_value=0;
char return_string[80];
int return_type;


sub_param(int idx,char *s)
{
}
sub_find(char *s,int *idx,int *zret, int *np, int **plist)
{
	*idx = 0;
	return 0;
}
sub_clear()
{
}
int sub_def(char *s)
{
}
sub_call(int idx,double *pval,char **pstr,int *npm)
{
	return;
}
sub_set_return(double d)
{
}
























                                                                    
