enum {f_plus=61 ,f_minus ,f_abs ,f_atn ,f_cos , f_date ,f_exp ,f_fix
		,f_int ,f_left ,f_len ,f_log ,f_log10 ,f_not
		,f_num ,f_num1 ,f_pos ,f_right ,f_rnd ,f_seg
		,f_sgn  ,f_sin ,f_sqr ,f_sqrt ,f_tan ,f_time
		,f_val  ,f_cell,f_miss, f_pow };
