#include "all.h"
#include "conio.h"
#include <math.h>
#include "bios.h"

int gle_debug;
int scr_init(void);
int scr_end(void);
#define true (!false)
#define false 0
#define dbg if (gle_debug>0)
#ifdef __TURBOC__
	extern unsigned _stklen=45000;
#endif

typedef char (*TEXT)[];
int int_edt(char *fname);
int scrinsert(int y);
int scrdeleteline(int y);
int text_eol(void);
int text_delete(void);
int text_undelete(void);
int text_deleteline(void);
int text_left(void);
int text_right(void);
int text_up(void);
int text_down(void);
int text_return(void);
int text_refresh();
int text_putstr(char *s);
int text_setwindow();
int text_move(int x,int y);
int scr_tab(char *s, int x);
int scr_negtab(char *s, int x, int *fpos, int *scrx);
int fner(char *s);
int fner_clear(void);
int text_setwindow(void);
int text_refresh(void);
int text_main(void);
int text_inkey(void);
int text_scroll(void);
int text_scroll_up(void);
int text_scroll_down(void);
int text_select(void);
int text_cut(void);
int text_paste(void);
int restofline(int y,int x);
int normal_key(int c);
int lineset(int i,char *ss);
int lineinsert(int y, char *ss);
int textinsert(int y);
char *sline(int i);
int text_load(char *f);
char *line(int i);
int ncpy(char *d, char *s, int n);
/*------------ GLOBAL VARIABLES --------------*/
char input_file[50];
char *(*gtxt)[];   /* gtxt is a pointer to an array of poiter to char */
int ngtxt=0;
/*---------------------------------------------------------------------------*/
main(int argc, char **argv)
{
char fname[30];
	if (argc>1) strcpy(fname,*(++argv)); else strcpy(fname,"test.gle");
	strcpy(input_file,fname);
	gtxt = (void *) myallocn(1,1000*sizeof(char *));	/* allocate 1000 lines of gle source  	4K */
	text_load(fname);
	scr_init();
	int_edt(fname);
	scr_end();
}


















