#include <ctype.h>
#include <string.h>
#include <assert.h>


#define false 0
#define true (!false)
extern int noscreenio;
void wprintf_do(char *s)
{
	if (noscreenio) printf("%s",s);
	else printw("%s",s);
}
