/* As varargs are so difficult to make work, I've put them
   in this module by themselfs,  on a DEC5400 it was necessary
   to compile this module using the native C compiler instead of
   GCC
*/



#include <varargs.h>

#ifndef MANIP
void gprint(va_alist)
va_dcl
{
        va_list args;
        char *fmt;
        char string[1024];

        va_start(args);
        fmt = va_arg(args, char *);
        vsprintf(string, fmt, args);
        va_end(args);
	gprint_do(string);
}
#else
void fner(va_alist)
va_dcl
{
        va_list args;
        char *fmt;
        char string[1024];

        va_start(args);
        fmt = va_arg(args, char *);
        vsprintf(string, fmt, args);
        va_end(args);
	fner_do(string);
}

void printmess(va_alist)
va_dcl
{
        va_list args;
        char *fmt;
        char string[1024];

        va_start(args);
        fmt = va_arg(args, char *);
        vsprintf(string, fmt, args);
        va_end(args);
	printmess_do(string);
}
#endif
void wprintf(va_alist)
va_dcl
{
        va_list args;
        char *fmt;
        char string[1024];

        va_start(args);
        fmt = va_arg(args, char *);
        vsprintf(string, fmt, args);
        va_end(args);
	wprintf_do(string);
}

