int d_dfont(char *c);
int d_message(char *s);
int d_source(char *s);
int d_get_type(char *t);
int d_set_path(int onoff);
int d_newpath(void);
int d_open(double width, double height);
int d_close(void);
int d_get_line_cap(int *i);
int d_set_line_cap(int i);
int d_set_line_join(int i);
int d_set_line_miterlimit(double d);
int d_set_line_width(double w);
int d_set_line_styled(double dd);
int d_set_line_style(char *s);
int d_fill(void);
int d_fill_ary(int nwk,double (*wkx)[],double (*wky)[]);
int d_line_ary(int nwk,double (*wkx)[],double (*wky)[]);
int d_stroke(void);
int d_clip(void);
int d_set_matrix(double newmat[3][3]);
int d_move(double zx,double zy);
int d_reverse(void) 	/* reverse the order of stuff in the current path */;
int d_closepath(void);
int d_line(double zx,double zy);
int d_clear(void);
int d_flush(void);
int d_arcto(dbl x1,dbl y1,dbl x2,dbl y2,dbl rrr);
int d_arc(dbl r,dbl t1,dbl t2,dbl cx,dbl cy);
int d_narc(dbl r,dbl t1,dbl t2,dbl cx,dbl cy);
int d_box_fill(dbl x1, dbl y1, dbl x2, dbl y2);
int d_box_stroke(dbl x1, dbl y1, dbl x2, dbl y2);
int xdbox(double x1, double y1, double x2, double y2);
int d_circle_stroke(double zr);
int d_circle_fill(double zr);
int d_bezier(dbl x1,dbl y1,dbl x2,dbl y2,dbl x3,dbl y3);
int set_color(void);
int set_fill(void);
int d_set_color(int32 f);
int d_set_fill(int32 f);
int d_beginclip(void) ;
int d_endclip(void) ;
int d_char(int font, int cc);

