/* some emulation routines for the calcomp plotting calls */
#include "int32.h"
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <assert.h>
#include <math.h>

#define true (!false)
#define false 0

/*main(int argc, char **argv)
*/

number(float *x, float *y, float *z, float *v, float *c1, int32 *c2)
{	
	printf("! Number %g %g %g {%g} %g %d \n",x,y,z,v,c1,c2);
	
}
symbol(float *x, float *y, float *z, char *s, float *c1, int32 *c2)
{	
	
}
plot(float *x1, float *y1, int *cmd)
{
	switch (*cmd) {
	  case 2:
		printf("aline %g %g \n",*x1,*y1);
		break;
	  case 3:
		printf("amove %g %g \n",*x1,*y1);
		break;
	  default:
		printf("! %d %g %g \n",*cmd,*x1,*y1);
		break;
	}
}

