extern HANDLE   hInst;
// Defines for menu item IDs
#define   IDM_QUIT                            101
#define   IDM_CONVERT                          102
#define   IDM_DISPLAY                          103
#define   IDM_HELP                            104
#define 	IDM_TEST			105

typedef char STRFIXED[200];

// Function prototypes for dialog and subdialog modules

