scr_refresh()
{
}
int scr_getch()
{
}
scr_init()
{
}
scr_end()
{
}
struct text_info r;
scr_savexy()
{
	gettextinfo(&r);
}
scr_restorexy()
{
	textattr(r.normattr);
	window(r.winleft,r.wintop,r.winright,r.winbottom);
	gotoxy(r.curx,r.cury);
}












