
	typedef union {
/*		struct {unsigned char b,g,r,f;} rgb ; */
		int32 l;
	        unsigned char b[4];
	} colortyp;
#ifdef BIGINDIAN
#define B_F 0
#define B_R 1
#define B_G 2
#define B_B 3
#else
#define B_F 3
#define B_R 2
#define B_G 1
#define B_B 0
#endif
