extern int32 *(*gpcode)[];   /* gpcode is a pointer to an array of poiter to int32 */
extern int32 (*gplen)[];   /* gpcode is a pointer to an array of int32 */
extern int ngpcode;

BEGINDEF char *tk[500];
BEGINDEF char strbuf[200];
BEGINDEF char srclin[200];
BEGINDEF char outbuff[300];
BEGINDEF char inbuff[300];
BEGINDEF char tkbuff[500];
BEGINDEF char space_str[3];
BEGINDEF int ntk;

int begin_token(int32 **pcode,int *cp,int *pln,char *srclin,char *tk,int *ntk,char *outbuff);
int begin_init(void);
int begin_next_line(int32 *pcode, int *cp);














