struct char_data { float wx,wy,x1,y1,x2,y2; };
struct xypair {float x,y;};
struct char_datax { uchar *kern,*lig,*lrep; struct xypair *krep; };
struct font_table {
	char *name;
	char *full_name;
	char *file_metric;
	char *file_vector;
	char *file_bitmap;
	struct char_data (*chr)[256];
	struct char_datax (*chrx)[256];

	int encoding;		/* 1 = postscript text , 2=postscrip symbol, */
				/* 3=TEX text, 4=TEX symbol, 5=TEX extensions */
	float space,space_stretch,space_shrink; /* after encoding *** */
	float scale,slant,uposition,uthickness;
	float fx1,fy1,fx2,fy2,caphei,xhei,descender,ascender;
	float fillerbug;
} ;

#ifdef FONTDEF
#else
#define FONTDEF
#endif

FONTDEF struct font_table fnt[100];
FONTDEF int nfnt;











