REM Run this file to install the makefiles for GCC/EMX and NMAKE OS/2

copy makefile.os2 makefile

cd font
copy makefile.os2 makefile

cd ..\util\contour 
copy makefile.os2 makefile

cd ..\fitls
copy makefile.os2 makefile

cd ..\fitz
copy makefile.os2 makefile

cd ..\letz
copy makefile.os2 makefile

cd ..\manip
copy makefile.os2 makefile

cd ..\surf 
copy makefile.os2 makefile

cd ..\..
