/*
 * memmove, copy overlapping strings
 *
 * We supply this routine for those systems that aren't standard yet.
 */

char *memmove(char *dest, char *src, int l)
{
	char *ret = dest;

	if (dest<src) {
		while (l--) *dest++ = *src++;
	} else {
		dest+=l; src+=l;
		while (l--) *(--dest) = *(--src);
	}
	return ret;
}
