#define YELLOW 1
#define BLACK 0
#define BLUE 0
#define LIGHTGRAY 0
#define WHITE 1
#define MAGENTA 1


struct text_info {
	int normattr,winleft,wintop,winright,winbottom,curx,cury;};
struct ffblk {char *ff_name;};















