@echo off
REM makgle32.bat by Axel Rohde
REM
REM Run this file to install the makefiles for gcc/djgpp/go32/GNUmake under DOS
REM

copy makefile.dj makefile
copy config.dj config.i

cd font
copy makefile.dj makefile

cd ..\util\contour 
copy makefile.dj makefile

cd ..\fitls
copy makefile.dj makefile

cd ..\fitz
copy makefile.dj makefile

cd ..\letz
copy makefile.dj makefile

cd ..\manip
copy makefile.dj makefile

cd ..\surf 
copy makefile.dj makefile

cd ..\..
