enum {eup=200,edown,eleft,eright,epagedown,epageup,edelline
	,edeleol,eescape,eendoffile,etopoffile,edelright
	,eselect,ecut,ecopy,emark,epaste,emove
	,eeol,ebol,eword,ebword,ereturn,etrace
	,eblockread,eblockwrite
	,ehelp,edelete,eundelline,esearch,esearchreplace,efindnext
	,esaveas,esave,eload,equit,edrawit,egraphmenu,eshowerror,egold
	,eundeleol,eexitnow,eshell,eedt,efast};



     
