int passt(int srclin, char *source,char (*(*tk)[500]),int *ntok,int32 *pcode,int *plen);
int gt_xy(int *curtok,char (*(*tk)[500]),int *ntok,int32 *pcode,int *plen);
int gt_optional(struct op_key (*lkey)[100],int *curtok, char (*(*tk)[500]),
	int *ntok, int32 *pcode, int *plen);
int mystrcpy(char **d, char *s);
int get_color(char (*(*tk)[500]), int *ntok, int *curtok,
	int32 *pcode, int *plen);
int get_fill(char (*(*tk)[500]), int *ntok, int *curtok,
	int32 *pcode, int *plen);
int get_justify(char (*(*tk)[500]), int *ntok, int *curtok,
	int32 *pcode, int *plen);
int get_marker(char (*(*tk)[500]), int *ntok, int *curtok,
	int32 *pcode, int *plen);
int spop(int v);
int spush(int v);
int scheck(int v);
int str_var(char *s);
int valid_var(char *s);



















