/* This routine builds the included part of the make file  config.i
    	cc -P config.cpp
	make
	make doutils 
 */

/* You can override this with the environment varialbe GLE_TOP, 
   be sure to put the trailing slash on. */

DEF_GLE_TOP = /usr/local/gle/

#ifdef ultrix
	CC = gcc
	CFLAGS = -w -Dunix
	LIBS = -lm -lcursesX -ltermcap -lc 
	XLIBS = -lm -lX11 -lcursesX -ltermcap -lc 
#endif
#ifdef sun
	MEMMOVE = memmove.o
	CC = gcc 
	CFLAGS = -w -Dunix -DNOKEYPAD -DNOATTRIB -DBIGINDIAN
	LIBS = -lm -lcurses -ltermcap -lc 
	XLIBS = -lm -lX11 -lcurses -ltermcap -lc 
#endif
#ifdef aix
	CC = xlc
	LIBS = 	-lm -lcurses -lc 
	XLIBS = -lm -lX11 -lcurses -lc 
	CFLAGS= -w -Dunix -DNOKEYPAD -DNOATTRIB -DBIGINDIAN
#endif
#ifdef xwindows
	XGLE = gle_x
	DXO = ../../d_x.o
	XSURF = surf_x
#endif

/* -ysymname to trace a symbol */





